import { writable } from "svelte/store";
import Toast from "../components/Toast"
import { openDB, DBSchema } from 'idb';


interface ImageDB extends DBSchema {
  images: {
    value: Image;
    key: number;
    indexes: { 'filename': string };
  };
}


const db = openDB<ImageDB>('Images', 1, {
  upgrade(db) {
    // Create a store of objects
    const store = db.createObjectStore('images', {
      // The 'id' property of the object will be the key.
      keyPath: 'id',
      // If it isn't explicitly set, create a value by auto incrementing.
      autoIncrement: true,
    });
    // Create an index on the 'date' property of the objects.
    store.createIndex('filename', 'filename');
  },
});



const store = writable<Image[]>([]);

async function add(img: Image | Image[]) {


  const images = Array.isArray(img) ? [...img] : [img];
  if (!images.length) return;

  const tx = (await db).transaction('images', 'readwrite');

  await Promise.all(images.map(img => {
    delete img.id;
    return tx.store.add(img);
  }));

  Toast.info("Loaded <b>" + images.length + "</b> images")

  store.set(await getAll())

  await tx.done;

}

async function updateImage(img: Image) {
  img.lastModified = Date.now();
  (await db).put("images", img);
  store.update(images => images.map(i => i.id === img.id ? img : i));
}

async function get(id: number): Promise<Image> {
  return (await db).get("images", id);
}

async function getAll(): Promise<Image[]> {
  return (await db).getAll("images");
}

async function deleteImage(img: Image): Promise<void> {
  (await db).delete("images", img.id);
  store.update(images => images.filter(i => i.id !== img.id));
}

(async () => {
  store.set(await getAll())
})()

export { store, add, get, getAll, updateImage, deleteImage };